/*
 * Copyright (c) 2015-2017 Carsten Sonne Larsen  <cs@innolan.dk>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 */

#ifndef _FUNCTION_GRID_H
#define _FUNCTION_GRID_H

/**
 * @file  fgrid.h
 * @brief Function grid used with graphs.
 *
 */

#include "lib/real.h"
#include "main/functions.h"

class Grid {
public:
    Grid(UserFunction* function);
    ~Grid();
    void SetFunctionBounderies(double minX, double maxX);
    void SetScreenBounderues(int minX, int maxX, int minY, int maxY);
    void GetScreenCoordinates(double value, int *x, int *y);
    void GetScreenCoordinates(double xval, int* x, double yval, int* y);
    double GetHorizontalResolution();

    void GetXAxis(int *xstart, int *xend, int *ystart, int *yend);
    void GetYAxis(int *xstart, int *xend, int *ystart, int *yend);

private:
    double FunctionValue(double parameter);

    UserFunction* function;
    RealNumber *parameter;

    double minX;
    double maxX;
    double minY;
    double maxY;

    int pad;
    int screenMinX;
    int screenMaxX;
    int screenMinY;
    int screenMaxY;

    int originX;
    int originY;
    double zoom;
    double scaleX;
    double scaleY;
};

#endif
